import { Plugin } from 'vite';
import * as sharp from 'sharp';
import { Sharp, FormatEnum } from 'sharp';
import { OutputAsset } from 'rollup';

declare function createImageApi(config: Config): {
    getImageById(id: string): sharp.Sharp;
    waitForImages(): Promise<OutputAsset[]>;
    resolveImage(filename: string, params: Record<string, any>): Promise<ImageResult>;
};

declare type ImageApi = ReturnType<typeof createImageApi>;
declare type Image = Sharp;
declare type ImageFormat = keyof FormatEnum;
declare type ImageFormats = {
    [k in ImageFormat]?: any;
};
declare type ImageAttrs = Partial<HTMLImageElement> & {
    class?: string;
};
declare type ImageResult = ImageAttrs[] & {
    src?: string;
};
declare type ImageGeneratorArgs = Record<string, any>;
declare type ImageGenerator = (image: Sharp, args: ImageGeneratorArgs) => Sharp;
interface ImageSpec {
    /**
     * A condition descriptor that specifies when the image should be used.
     * Can be a width descriptor or a density descriptor.
     * https://developer.mozilla.org/en-US/docs/Web/API/HTMLImageElement/srcset
     */
    condition?: string;
    /**
     * A function to generate the image.
     */
    generate: ImageGenerator;
    /**
     * The parameters for the image generation function.
     */
    args: ImageGeneratorArgs;
}
interface ImageSource {
    type?: string;
    media?: string;
    sizes?: string;
    srcset: ImageSpec[];
}
interface ImagePreset {
    attrs?: ImageAttrs;
    images: ImageSource[];
}
declare type ImagePresets = Record<string, ImagePreset>;
interface Options {
    /**
     * The directory in which to place processed images, relative to Vite's `outDir`.
     * @default 'assets/images'
     */
    assetsDir?: string;
    /**
     * The directory to use for caching images between builds.
     * @default 'node_modules/.images'
     */
    cacheDir?: string;
    /**
     * Definitions of image presets to apply.
     */
    presets?: ImagePresets;
    /**
     * URL parameter that specifies the image preset.
     * @default 'preset'
     */
    urlParam?: string;
}
interface Config extends Required<Options> {
    isBuild: boolean;
    outDir: string;
    base: string;
    root: string;
}

declare function formatFor(image: Image): ImageFormat;
declare function mimeTypeFor(format: ImageFormat): string;

interface WidthPresetOptions extends ImageAttrs {
    widths: number[];
    formats: ImageFormats;
}

declare function widthPreset({ widths, formats, ...attrs }: WidthPresetOptions): ImagePreset;

declare function ImagePresetsPlugin(presets?: ImagePresets, options?: Options): Plugin & {
    api: ImageApi;
};

export { Config, Image, ImageApi, ImageAttrs, ImageFormat, ImageFormats, ImageGenerator, ImageGeneratorArgs, ImagePreset, ImagePresets, ImageResult, ImageSource, ImageSpec, Options, ImagePresetsPlugin as default, formatFor, mimeTypeFor, widthPreset };
